@@ -425,7 +425,7 @@ gst_element_set_clock (GstElement * element, GstClock * clock)
 
   oclass = GST_ELEMENT_GET_CLASS (element);
 
-  GST_DEBUG_OBJECT (element, "setting clock %p", clock);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, element, "setting clock %p", clock);
 
   if (oclass->set_clock)
     res = oclass->set_clock (element, clock);
@@ -478,14 +478,18 @@ gst_element_get_clock (GstElement * element)
 void
 gst_element_set_base_time (GstElement * element, GstClockTime time)
 {
+  GstClockTime old;
+
   g_return_if_fail (GST_IS_ELEMENT (element));
 
   GST_OBJECT_LOCK (element);
+  old = element->base_time;
   element->base_time = time;
   GST_OBJECT_UNLOCK (element);
 
-  GST_DEBUG_OBJECT (element, "set base_time=%" GST_TIME_FORMAT,
-      GST_TIME_ARGS (time));
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, element,
+      "set base_time=%" GST_TIME_FORMAT ", old %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (time), GST_TIME_ARGS (old));
 }
 
 /**
@@ -1552,7 +1556,8 @@ gst_element_post_message (GstElement * element, GstMessage * message)
   /* ERRORS */
 no_bus:
   {
-    GST_DEBUG_OBJECT (element, "not posting message %p: no bus", message);
+    GST_CAT_DEBUG_OBJECT (GST_CAT_MESSAGE, element,
+        "not posting message %p: no bus", message);
     GST_OBJECT_UNLOCK (element);
     gst_message_unref (message);
     return FALSE;
@@ -2295,12 +2300,15 @@ gst_element_set_state_func (GstElement * element, GstState state)
   current = GST_STATE (element);
   next = GST_STATE_NEXT (element);
   old_pending = GST_STATE_PENDING (element);
-  /* increment state cookie so that we can track each state change */
-  element->state_cookie++;
 
   /* this is the (new) state we should go to. TARGET is the last state we set on
    * the element. */
-  GST_STATE_TARGET (element) = state;
+  if (state != GST_STATE_TARGET (element)) {
+    GST_STATE_TARGET (element) = state;
+    /* increment state cookie so that we can track each state change. We only do
+     * this if this is actually a new state change. */
+    element->state_cookie++;
+  }
   GST_STATE_PENDING (element) = state;
 
   GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, element,
@@ -2344,7 +2352,7 @@ gst_element_set_state_func (GstElement * element, GstState state)
       (next != state ? "intermediate" : "final"),
       gst_element_state_get_name (current), gst_element_state_get_name (next));
 
-  /* now signal any waiters, they will error since the cookie was increased */
+  /* now signal any waiters, they will error since the cookie was incremented */
   GST_STATE_BROADCAST (element);
 
   GST_OBJECT_UNLOCK (element);
@@ -2547,7 +2555,8 @@ gst_element_pads_activate (GstElement * element, gboolean active)
   GstIterator *iter;
   gboolean res;
 
-  GST_DEBUG_OBJECT (element, "pads_activate with active %d", active);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_ELEMENT_PADS, element,
+      "pads_activate with active %d", active);
 
   iter = gst_element_iterate_src_pads (element);
   res =
@@ -2576,7 +2585,8 @@ gst_element_pads_activate (GstElement * element, gboolean active)
       goto caps_failed;
   }
 
-  GST_DEBUG_OBJECT (element, "pads_activate successful");
+  GST_CAT_DEBUG_OBJECT (GST_CAT_ELEMENT_PADS, element,
+      "pads_activate successful");
 
   return TRUE;
 
@@ -2696,7 +2706,8 @@ gst_element_dispose (GObject * object)
   if (GST_STATE (element) != GST_STATE_NULL)
     goto not_null;
 
-  GST_DEBUG_OBJECT (element, "removing %d pads", g_list_length (element->pads));
+  GST_CAT_DEBUG_OBJECT (GST_CAT_ELEMENT_PADS, element,
+      "removing %d pads", g_list_length (element->pads));
   /* first we break all our links with the outside */
   while (element->pads && element->pads->data) {
     /* don't call _remove_pad with NULL */
@@ -2950,7 +2961,8 @@ gst_element_get_bus (GstElement * element)
     gst_object_ref (result);
   GST_OBJECT_UNLOCK (element);
 
-  GST_DEBUG_OBJECT (element, "got bus %" GST_PTR_FORMAT, result);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_BUS, element, "got bus %" GST_PTR_FORMAT,
+      result);
 
   return result;
 }
