@@ -1,7 +1,7 @@
 /* GStreamer
  * Copyright (C) 2006 Josep Torra <josep@fluendo.com>
  *               2006 Mathieu Garcia <matthieu@fluendo.com>
- *		         2006,2007 Stefan Kost <ensonic@users.sf.net>
+ *               2006,2007 Stefan Kost <ensonic@users.sf.net>
  *
  * gstregistrybinary.c: GstRegistryBinary object, support routines
  *
@@ -65,6 +65,8 @@
 
 #include <glib/gstdio.h>        /* for g_stat(), g_mapped_file(), ... */
 
+#include "glib-compat-private.h"
+
 
 #define GST_CAT_DEFAULT GST_CAT_REGISTRY
 
@@ -74,6 +76,10 @@
   _outptr = (_element *)_inptr; \
   _inptr += sizeof (_element)
 
+#define unpack_const_string(_inptr, _outptr) \
+  _outptr = g_intern_string ((const gchar *)_inptr); \
+  _inptr += strlen(_outptr) + 1
+
 #define unpack_string(_inptr, _outptr)  \
   _outptr = g_strdup ((gchar *)_inptr); \
   _inptr += strlen(_outptr) + 1
@@ -151,12 +157,12 @@ gst_registry_binary_initialize_magic (GstBinaryRegistryMagic * m)
  * Returns: %TRUE for success
  */
 inline static gboolean
-gst_registry_binary_save_const_string (GList ** list, gchar * str)
+gst_registry_binary_save_const_string (GList ** list, const gchar * str)
 {
   GstBinaryChunk *chunk;
 
   chunk = g_malloc (sizeof (GstBinaryChunk));
-  chunk->data = str;
+  chunk->data = (gpointer) str;
   chunk->size = strlen ((gchar *) chunk->data) + 1;
   chunk->flags = GST_BINARY_REGISTRY_FLAG_CONST;
   chunk->align = FALSE;
@@ -538,16 +544,17 @@ gst_registry_binary_write_cache (GstRegistry * registry, const char *location)
   }
   g_list_free (to_write);
 
-  if (close (registry->cache_file) < 0) {
-    GST_DEBUG ("Can't close registry file : %s", g_strerror (errno));
-    goto fail;
-  }
+  if (close (registry->cache_file) < 0)
+    goto close_failed;
 
   if (g_file_test (tmp_location, G_FILE_TEST_EXISTS)) {
 #ifdef WIN32
-    remove (location);
+    g_remove (location);
 #endif
-    rename (tmp_location, location);
+    if (g_rename (tmp_location, location) < 0)
+      goto rename_failed;
+  } else {
+    /* FIXME: shouldn't we return FALSE here? */
   }
 
   g_free (tmp_location);
@@ -556,8 +563,26 @@ gst_registry_binary_write_cache (GstRegistry * registry, const char *location)
 
   /* Errors */
 fail:
-  g_free (tmp_location);
-  return FALSE;
+  {
+    (void) close (registry->cache_file);
+    /* fall through */
+  }
+fail_after_close:
+  {
+    g_remove (tmp_location);
+    g_free (tmp_location);
+    return FALSE;
+  }
+close_failed:
+  {
+    GST_ERROR ("close() failed: %s", g_strerror (errno));
+    goto fail_after_close;
+  }
+rename_failed:
+  {
+    GST_ERROR ("g_rename() failed: %s", g_strerror (errno));
+    goto fail_after_close;
+  }
 }
 
 
@@ -624,7 +649,7 @@ gst_registry_binary_load_pad_template (GstElementFactory * factory, gchar ** in)
   template->direction = pt->direction;
 
   /* unpack pad template strings */
-  unpack_string (*in, template->name_template);
+  unpack_const_string (*in, template->name_template);
   unpack_string (*in, template->static_caps.string);
 
   __gst_element_factory_add_static_pad_template (factory, template);
@@ -643,7 +668,7 @@ gst_registry_binary_load_pad_template (GstElementFactory * factory, gchar ** in)
  */
 static gboolean
 gst_registry_binary_load_feature (GstRegistry * registry, gchar ** in,
-    gchar * plugin_name)
+    const gchar * plugin_name)
 {
   GstBinaryPluginFeature *pf = NULL;
   GstPluginFeature *feature;
@@ -762,7 +787,9 @@ gst_registry_binary_load_feature (GstRegistry * registry, gchar ** in,
 #endif
 
   feature->rank = pf->rank;
-  feature->plugin_name = plugin_name;
+
+  /* should already be the interned string, but better make sure */
+  feature->plugin_name = g_intern_string (plugin_name);
 
   gst_registry_add_feature (registry, feature);
   GST_DEBUG ("Added feature %s", feature->name);
@@ -817,14 +844,14 @@ gst_registry_binary_load_plugin (GstRegistry * registry, gchar ** in)
   plugin->file_size = pe->file_size;
 
   /* unpack plugin element strings */
-  unpack_string (*in, plugin->desc.name);
+  unpack_const_string (*in, plugin->desc.name);
   unpack_string (*in, plugin->desc.description);
   unpack_string (*in, plugin->filename);
-  unpack_string (*in, plugin->desc.version);
-  unpack_string (*in, plugin->desc.license);
-  unpack_string (*in, plugin->desc.source);
-  unpack_string (*in, plugin->desc.package);
-  unpack_string (*in, plugin->desc.origin);
+  unpack_const_string (*in, plugin->desc.version);
+  unpack_const_string (*in, plugin->desc.license);
+  unpack_const_string (*in, plugin->desc.source);
+  unpack_const_string (*in, plugin->desc.package);
+  unpack_const_string (*in, plugin->desc.origin);
   GST_LOG ("read strings for '%s'", plugin->desc.name);
 
   plugin->basename = g_path_get_basename (plugin->filename);
@@ -832,8 +859,7 @@ gst_registry_binary_load_plugin (GstRegistry * registry, gchar ** in)
   GST_INFO ("Added plugin '%s' plugin with %d features from binary registry",
       plugin->desc.name, pe->nfeatures);
   for (i = 0; i < pe->nfeatures; i++) {
-    if (!gst_registry_binary_load_feature (registry, in,
-            g_strdup (plugin->desc.name))) {
+    if (!gst_registry_binary_load_feature (registry, in, plugin->desc.name)) {
       GST_ERROR ("Error while loading binary feature");
       goto fail;
     }
@@ -887,6 +913,7 @@ gst_registry_binary_read_cache (GstRegistry * registry, const char *location)
     g_file_get_contents (location, &contents, &size, &err);
     if (err != NULL) {
       GST_INFO ("Unable to read file %s : %s", location, err->message);
+      g_timer_destroy (timer);
       g_error_free (err);
       return FALSE;
     }
@@ -935,7 +962,6 @@ gst_registry_binary_read_cache (GstRegistry * registry, const char *location)
 
   g_timer_stop (timer);
   seconds = g_timer_elapsed (timer, NULL);
-  g_timer_destroy (timer);
 
   GST_INFO ("loaded %s in %lf seconds", location, seconds);
 
@@ -943,6 +969,7 @@ gst_registry_binary_read_cache (GstRegistry * registry, const char *location)
   /* TODO: once we re-use the pointers to registry contents return here */
 
 Error:
+  g_timer_destroy (timer);
   if (mapped) {
     g_mapped_file_free (mapped);
   } else {
