@@ -24,29 +24,32 @@
  * @short_description: Generic structure containing fields of names and values
  * @see_also: #GstCaps, #GstMessage, #GstEvent, #GstQuery
  *
- * A #GstStructure is a collection of key/value pairs. The keys are expressed as
- * GQuarks and the values can be of any GType.
+ * A #GstStructure is a collection of key/value pairs. The keys are expressed
+ * as GQuarks and the values can be of any GType.
  *
- * In addition to the key/value pairs, a #GstStructure also has a name.
+ * In addition to the key/value pairs, a #GstStructure also has a name. The name
+ * starts with a letter and can be folled by letters, numbers and any of "/-_.:".
  * 
- * #GstStructure is used by various GStreamer subsystems to store information in
- * a flexible an extensible way. A #GstStructure does not have a refcount because it
- * usually is part of a higher level object such as #GstCaps. It provides a means to 
- * enforce mutability using the refcount of the parent with the 
- * gst_structure_set_parent_refcount() method.
+ * #GstStructure is used by various GStreamer subsystems to store information
+ * in a flexible and extensible way. A #GstStructure does not have a refcount
+ * because it usually is part of a higher level object such as #GstCaps. It
+ * provides a means to enforce mutability using the refcount of the parent
+ * with the gst_structure_set_parent_refcount() method.
  *
- * A #GstStructure can be created with gst_structure_empty_new() or gst_structure_new(),
- * which both take a name and an optional set of key/value pairs along with the types
- * of the values.
+ * A #GstStructure can be created with gst_structure_empty_new() or
+ * gst_structure_new(), which both take a name and an optional set of
+ * key/value pairs along with the types of the values.
  * 
- * Field values can be changed with gst_structure_set_value() or gst_structure_set().
+ * Field values can be changed with gst_structure_set_value() or
+ * gst_structure_set().
  *
- * Field values can be retrieved with gst_structure_get_value() or the more convenient
- * gst_structure_get_*() functions.
+ * Field values can be retrieved with gst_structure_get_value() or the more
+ * convenient gst_structure_get_*() functions.
  *
- * Fields can be removed with gst_structure_remove_field() or gst_structure_remove_fields().
+ * Fields can be removed with gst_structure_remove_field() or
+ * gst_structure_remove_fields().
  *
- * Last reviewed on 2005-11-09 (0.9.4)
+ * Last reviewed on 2007-10-16 (0.10.15)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -135,18 +138,48 @@ gst_structure_id_empty_new (GQuark quark)
   return gst_structure_id_empty_new_with_size (quark, 0);
 }
 
+static gboolean
+gst_structure_validate_name (const gchar * name)
+{
+  const gchar *s;
+
+  g_return_val_if_fail (name != NULL, FALSE);
+
+  /* FIXME 0.11: use g_ascii_isalpha() */
+  if (!g_ascii_isalnum (*name)) {
+    GST_WARNING ("Invalid character '%c' at offset 0 in structure name: %s",
+        *name, name);
+    return FALSE;
+  }
+
+  /* FIXME 0.11: don't allow spaces */
+  /* FIXME: test name string more */
+  s = &name[1];
+  while (*s && (g_ascii_isalnum (*s) || strchr ("/-_.:+ ", *s) != NULL))
+    s++;
+  if (*s != '\0') {
+    GST_WARNING ("Invalid character '%c' at offset %lu in structure name: %s",
+        *s, ((gulong) s - (gulong) name), name);
+    return FALSE;
+  }
+
+  return TRUE;
+}
+
 /**
  * gst_structure_empty_new:
  * @name: name of new structure
  *
- * Creates a new, empty #GstStructure with the given name.
+ * Creates a new, empty #GstStructure with the given @name.
+ *
+ * See gst_structure_set_name() for constraints on the @name parameter.
  *
  * Returns: a new, empty #GstStructure
  */
 GstStructure *
 gst_structure_empty_new (const gchar * name)
 {
-  g_return_val_if_fail (name != NULL, NULL);
+  g_return_val_if_fail (gst_structure_validate_name (name), NULL);
 
   return gst_structure_id_empty_new_with_size (g_quark_from_string (name), 0);
 }
@@ -187,9 +220,11 @@ gst_structure_new (const gchar * name, const gchar * firstfield, ...)
  * @firstfield: name of first field to set
  * @varargs: variable argument list
  *
- * Creates a new #GstStructure with the given name.  Structure fields
+ * Creates a new #GstStructure with the given @name.  Structure fields
  * are set according to the varargs in a manner similar to
- * @gst_structure_new.
+ * gst_structure_new().
+ *
+ * See gst_structure_set_name() for constraints on the @name parameter.
  *
  * Returns: a new #GstStructure
  */
@@ -202,7 +237,9 @@ gst_structure_new_valist (const gchar * name,
   g_return_val_if_fail (name != NULL, NULL);
 
   structure = gst_structure_empty_new (name);
-  gst_structure_set_valist (structure, firstfield, varargs);
+
+  if (structure)
+    gst_structure_set_valist (structure, firstfield, varargs);
 
   return structure;
 }
@@ -329,6 +366,9 @@ gst_structure_has_name (const GstStructure * structure, const gchar * name)
   g_return_val_if_fail (structure != NULL, FALSE);
   g_return_val_if_fail (name != NULL, FALSE);
 
+  /* getting the string is cheap and comparing short strings is too
+   * should be faster than getting the quark for name and comparing the quarks
+   */
   structure_name = g_quark_to_string (structure->name);
 
   return (structure_name && strcmp (structure_name, name) == 0);
@@ -355,15 +395,16 @@ gst_structure_get_name_id (const GstStructure * structure)
  * @structure: a #GstStructure
  * @name: the new name of the structure
  *
- * Sets the name of the structure to the given name.  The string
- * provided is copied before being used.
+ * Sets the name of the structure to the given @name.  The string
+ * provided is copied before being used. It must not be empty, start with a
+ * letter and can be followed by letters, numbers and any of "/-_.:".
  */
 void
 gst_structure_set_name (GstStructure * structure, const gchar * name)
 {
   g_return_if_fail (structure != NULL);
-  g_return_if_fail (name != NULL);
   g_return_if_fail (IS_MUTABLE (structure));
+  g_return_if_fail (gst_structure_validate_name (name));
 
   structure->name = g_quark_from_string (name);
 }
@@ -388,19 +429,16 @@ gst_structure_id_set_value (GstStructure * structure,
   g_return_if_fail (G_IS_VALUE (value));
   g_return_if_fail (IS_MUTABLE (structure));
 
-  /* if someones disables GST_DEBUG output, they probably do it for
-   * performance reasons, so skip the UTF-8 check here as well then */
-#ifndef GST_DISABLE_GST_DEBUG
   if (G_VALUE_HOLDS_STRING (value)) {
     const gchar *s;
 
     s = g_value_get_string (value);
-    if (s != NULL && !g_utf8_validate (s, -1, NULL)) {
+    if (G_UNLIKELY (s != NULL && !g_utf8_validate (s, -1, NULL))) {
       g_warning ("Trying to set string field '%s' on structure, but string is "
           "not valid UTF-8. Please file a bug.", g_quark_to_string (field));
+      return;
     }
   }
-#endif
 
   gsfield.name = field;
   gst_value_init_and_copy (&gsfield.value, value);
@@ -480,12 +518,10 @@ gst_structure_set_valist (GstStructure * structure,
 
     type = va_arg (varargs, GType);
 
-#if GLIB_CHECK_VERSION(2,8,0)
     if (type == G_TYPE_DATE) {
       g_warning ("Don't use G_TYPE_DATE, use GST_TYPE_DATE instead\n");
       type = GST_TYPE_DATE;
     }
-#endif
 
     g_value_init (&field.value, type);
     G_VALUE_COLLECT (&field.value, varargs, 0, &err);
@@ -499,6 +535,76 @@ gst_structure_set_valist (GstStructure * structure,
   }
 }
 
+/**
+ * gst_structure_id_set:
+ * @structure: a #GstStructure
+ * @fieldname: the GQuark for the name of the field to set
+ * @...: variable arguments
+ *
+ * Identical to gst_structure_set, except that field names are
+ * passed using the GQuark for the field name. This allows more efficient
+ * setting of the structure if the caller already knows the associated
+ * quark values.
+ * The last variable argument must be NULL.
+ *
+ * Since: 0.10.10
+ */
+void
+gst_structure_id_set (GstStructure * structure, GQuark field, ...)
+{
+  va_list varargs;
+
+  g_return_if_fail (structure != NULL);
+
+  va_start (varargs, field);
+  gst_structure_id_set_valist (structure, field, varargs);
+  va_end (varargs);
+}
+
+/**
+ * gst_structure_id_set_valist:
+ * @structure: a #GstStructure
+ * @fieldname: the name of the field to set
+ * @varargs: variable arguments
+ *
+ * va_list form of gst_structure_id_set().
+ *
+ * Since: 0.10.10
+ */
+void
+gst_structure_id_set_valist (GstStructure * structure,
+    GQuark fieldname, va_list varargs)
+{
+  gchar *err = NULL;
+  GType type;
+
+  g_return_if_fail (structure != NULL);
+  g_return_if_fail (IS_MUTABLE (structure));
+
+  while (fieldname) {
+    GstStructureField field = { 0 };
+
+    field.name = fieldname;
+
+    type = va_arg (varargs, GType);
+
+    if (type == G_TYPE_DATE) {
+      g_warning ("Don't use G_TYPE_DATE, use GST_TYPE_DATE instead\n");
+      type = GST_TYPE_DATE;
+    }
+
+    g_value_init (&field.value, type);
+    G_VALUE_COLLECT (&field.value, varargs, 0, &err);
+    if (err) {
+      g_critical ("%s", err);
+      return;
+    }
+    gst_structure_set_field (structure, &field);
+
+    fieldname = va_arg (varargs, GQuark);
+  }
+}
+
 /* If the structure currently contains a field with the same name, it is
  * replaced with the provided field. Otherwise, the field is added to the
  * structure. The field's value is not deeply copied.
@@ -770,7 +876,11 @@ gst_structure_nth_field_name (const GstStructure * structure, guint index)
 {
   GstStructureField *field;
 
+  g_return_val_if_fail (structure != NULL, NULL);
+  g_return_val_if_fail (index < structure->fields->len, NULL);
+
   field = GST_STRUCTURE_FIELD (structure, index);
+
   return g_quark_to_string (field->name);
 }
 
@@ -795,6 +905,7 @@ gst_structure_foreach (const GstStructure * structure,
   gboolean ret;
 
   g_return_val_if_fail (structure != NULL, FALSE);
+  g_return_val_if_fail (func != NULL, FALSE);
 
   for (i = 0; i < structure->fields->len; i++) {
     field = GST_STRUCTURE_FIELD (structure, i);
@@ -814,8 +925,8 @@ gst_structure_foreach (const GstStructure * structure,
  * @user_data: private data
  *
  * Calls the provided function once for each field in the #GstStructure. In
- * contrast to gst_structure_foreach(), the function may modify the fields. The
- * structure must be mutable.
+ * contrast to gst_structure_foreach(), the function may modify but not delete the
+ * fields. The structure must be mutable.
  *
  * Returns: TRUE if the supplied function returns TRUE For each of the fields,
  * FALSE otherwise.
@@ -830,6 +941,7 @@ gst_structure_map_in_place (GstStructure * structure,
 
   g_return_val_if_fail (structure != NULL, FALSE);
   g_return_val_if_fail (IS_MUTABLE (structure), FALSE);
+  g_return_val_if_fail (func != NULL, FALSE);
 
   for (i = 0; i < structure->fields->len; i++) {
     field = GST_STRUCTURE_FIELD (structure, i);
@@ -905,8 +1017,8 @@ gst_structure_has_field_typed (const GstStructure * structure,
  * and has the correct type.
  *
  * Returns: TRUE if the value could be set correctly. If there was no field
- * with @fieldname or the existing field did not contain a boolean, this function
- * returns FALSE.
+ * with @fieldname or the existing field did not contain a boolean, this
+ * function returns FALSE.
  */
 gboolean
 gst_structure_get_boolean (const GstStructure * structure,
@@ -939,9 +1051,9 @@ gst_structure_get_boolean (const GstStructure * structure,
  * given field.  Caller is responsible for making sure the field exists
  * and has the correct type.
  *
- * Returns: TRUE if the value could be set correctly. If there was no field
+ * Returns: %TRUE if the value could be set correctly. If there was no field
  * with @fieldname or the existing field did not contain an int, this function
- * returns FALSE.
+ * returns %FALSE.
  */
 gboolean
 gst_structure_get_int (const GstStructure * structure,
@@ -965,6 +1077,44 @@ gst_structure_get_int (const GstStructure * structure,
   return TRUE;
 }
 
+/**
+ * gst_structure_get_uint:
+ * @structure: a #GstStructure
+ * @fieldname: the name of a field
+ * @value: a pointer to a uint to set
+ *
+ * Sets the uint pointed to by @value corresponding to the value of the
+ * given field.  Caller is responsible for making sure the field exists
+ * and has the correct type.
+ *
+ * Returns: %TRUE if the value could be set correctly. If there was no field
+ * with @fieldname or the existing field did not contain a uint, this function
+ * returns %FALSE.
+ *
+ * Since: 0.10.15
+ */
+gboolean
+gst_structure_get_uint (const GstStructure * structure,
+    const gchar * fieldname, guint * value)
+{
+  GstStructureField *field;
+
+  g_return_val_if_fail (structure != NULL, FALSE);
+  g_return_val_if_fail (fieldname != NULL, FALSE);
+  g_return_val_if_fail (value != NULL, FALSE);
+
+  field = gst_structure_get_field (structure, fieldname);
+
+  if (field == NULL)
+    return FALSE;
+  if (!G_VALUE_HOLDS_UINT (&field->value))
+    return FALSE;
+
+  *value = g_value_get_uint (&field->value);
+
+  return TRUE;
+}
+
 /**
  * gst_structure_get_fourcc:
  * @structure: a #GstStructure
@@ -1229,6 +1379,8 @@ typedef struct _GstStructureAbbreviation
 }
 GstStructureAbbreviation;
 
+/* return a copy of an array of GstStructureAbbreviation containing all the
+ * known type_string, GType maps, including abbreviations for common types */
 static GstStructureAbbreviation *
 gst_structure_get_abbrs (gint * n_abbrs)
 {
@@ -1269,6 +1421,8 @@ gst_structure_get_abbrs (gint * n_abbrs)
       {"str", G_TYPE_STRING}
       ,
       {"s", G_TYPE_STRING}
+      ,
+      {"structure", GST_TYPE_STRUCTURE}
     };
     num = G_N_ELEMENTS (dyn_abbrs);
     /* permanently allocate and copy the array now */
@@ -1280,8 +1434,10 @@ gst_structure_get_abbrs (gint * n_abbrs)
   return abbrs;
 }
 
+/* given a type_name that could be a type abbreviation or a registered GType,
+ * return a matching GType */
 static GType
-gst_structure_from_abbr (const char *type_name)
+gst_structure_gtype_from_abbr (const char *type_name)
 {
   int i;
   GstStructureAbbreviation *abbrs;
@@ -1345,10 +1501,44 @@ gst_structure_value_get_generic_type (GValue * val)
   return G_VALUE_TYPE (val);
 }
 
+/* keep in sync with gstvalue.c */
 #define GST_ASCII_IS_STRING(c) (g_ascii_isalnum((c)) || ((c) == '_') || \
       ((c) == '-') || ((c) == '+') || ((c) == '/') || ((c) == ':') || \
       ((c) == '.'))
 
+gboolean
+priv_gst_structure_append_to_gstring (const GstStructure * structure,
+    GString * s)
+{
+  GstStructureField *field;
+  guint i;
+
+  g_return_val_if_fail (s != NULL, FALSE);
+
+  g_string_append (s, g_quark_to_string (structure->name));
+  for (i = 0; i < structure->fields->len; i++) {
+    char *t;
+    GType type;
+
+    field = GST_STRUCTURE_FIELD (structure, i);
+
+    t = gst_value_serialize (&field->value);
+    type = gst_structure_value_get_generic_type (&field->value);
+
+    g_string_append_len (s, ", ", 2);
+    /* FIXME: do we need to escape fieldnames? */
+    g_string_append (s, g_quark_to_string (field->name));
+    g_string_append_len (s, "=(", 2);
+    g_string_append (s, gst_structure_to_abbr (type));
+    g_string_append_c (s, ')');
+    g_string_append (s, GST_STR_NULL (t));
+    g_free (t);
+  }
+
+  g_string_append_c (s, ';');
+  return TRUE;
+}
+
 /**
  * gst_structure_to_string:
  * @structure: a #GstStructure
@@ -1361,9 +1551,7 @@ gst_structure_value_get_generic_type (GValue * val)
 gchar *
 gst_structure_to_string (const GstStructure * structure)
 {
-  GstStructureField *field;
   GString *s;
-  guint i;
 
   /* NOTE:  This function is potentially called by the debug system,
    * so any calls to gst_log() (and GST_DEBUG(), GST_LOG(), etc.)
@@ -1373,22 +1561,10 @@ gst_structure_to_string (const GstStructure * structure)
 
   g_return_val_if_fail (structure != NULL, NULL);
 
-  s = g_string_new ("");
-  /* FIXME this string may need to be escaped */
-  g_string_append_printf (s, "%s", g_quark_to_string (structure->name));
-  for (i = 0; i < structure->fields->len; i++) {
-    char *t;
-    GType type;
-
-    field = GST_STRUCTURE_FIELD (structure, i);
-
-    t = gst_value_serialize (&field->value);
-    type = gst_structure_value_get_generic_type (&field->value);
-
-    g_string_append_printf (s, ", %s=(%s)%s", g_quark_to_string (field->name),
-        gst_structure_to_abbr (type), GST_STR_NULL (t));
-    g_free (t);
-  }
+  /* we estimate a minimum size based on the number of fields in order to
+   * avoid unnecessary reallocs within GString */
+  s = g_string_sized_new (STRUCTURE_ESTIMATED_STRING_LEN (structure));
+  priv_gst_structure_append_to_gstring (structure, s);
   return g_string_free (s, FALSE);
 }
 
@@ -1447,7 +1623,6 @@ gst_structure_parse_range (gchar * s, gchar ** after, GValue * value,
   GType range_type;
   gboolean ret;
 
-
   if (*s != '[')
     return FALSE;
   s++;
@@ -1642,6 +1817,7 @@ gst_structure_parse_value (gchar * str,
   while (g_ascii_isspace (*s))
     s++;
 
+  /* check if there's a (type_name) 'cast' */
   type_name = NULL;
   if (*s == '(') {
     type = G_TYPE_INVALID;
@@ -1663,7 +1839,7 @@ gst_structure_parse_value (gchar * str,
 
     c = *type_end;
     *type_end = 0;
-    type = gst_structure_from_abbr (type_name);
+    type = gst_structure_gtype_from_abbr (type_name);
     *type_end = c;
 
     if (type == G_TYPE_INVALID)
@@ -1687,10 +1863,12 @@ gst_structure_parse_value (gchar * str,
     *value_end = 0;
     if (type == G_TYPE_INVALID) {
       GType try_types[] =
-          { G_TYPE_INT, G_TYPE_DOUBLE, GST_TYPE_FRACTION, G_TYPE_STRING };
+          { G_TYPE_INT, G_TYPE_DOUBLE, GST_TYPE_FRACTION, G_TYPE_BOOLEAN,
+        G_TYPE_STRING
+      };
       int i;
 
-      for (i = 0; i < 4; i++) {
+      for (i = 0; i < G_N_ELEMENTS (try_types); i++) {
         g_value_init (value, try_types[i]);
         ret = gst_value_deserialize (value, value_s);
         if (ret)
@@ -1720,7 +1898,7 @@ gst_structure_parse_value (gchar * str,
  * where parsing ended will be returned.
  *
  * Returns: a new #GstStructure or NULL when the string could not
- * be parsed. Free after usage.
+ * be parsed. Free with gst_structure_free() after use.
  */
 GstStructure *
 gst_structure_from_string (const gchar * string, gchar ** end)
@@ -1738,23 +1916,42 @@ gst_structure_from_string (const gchar * string, gchar ** end)
   copy = g_strdup (string);
   r = copy;
 
-  name = r;
-  if (!gst_structure_parse_string (r, &w, &r))
-    goto error;
-
-  while (g_ascii_isspace (*r) || (r[0] == '\\' && g_ascii_isspace (r[1])))
+  /* skip spaces (FIXME: _isspace treats tabs and newlines as space!) */
+  while (*r && (g_ascii_isspace (*r) || (r[0] == '\\'
+              && g_ascii_isspace (r[1]))))
     r++;
-  if (*r != 0 && *r != ';' && *r != ',')
+
+  name = r;
+  if (!gst_structure_parse_string (r, &w, &r)) {
+    GST_WARNING ("Failed to parse structure string");
     goto error;
+  }
 
   save = *w;
   *w = 0;
   structure = gst_structure_empty_new (name);
   *w = save;
 
-  while (*r && (*r != ';')) {
-    if (*r != ',')
+  if (structure == NULL)
+    goto error;
+
+  do {
+    while (*r && (g_ascii_isspace (*r) || (r[0] == '\\'
+                && g_ascii_isspace (r[1]))))
+      r++;
+    if (*r == ';') {
+      /* end of structure, get the next char and finish */
+      r++;
+      break;
+    }
+    if (*r == '\0') {
+      /* accept \0 as end delimiter */
+      break;
+    }
+    if (*r != ',') {
+      GST_WARNING ("Failed to find delimiter, r=%s", r);
       goto error;
+    }
     r++;
     while (*r && (g_ascii_isspace (*r) || (r[0] == '\\'
                 && g_ascii_isspace (r[1]))))
@@ -1764,10 +1961,8 @@ gst_structure_from_string (const gchar * string, gchar ** end)
     if (!gst_structure_parse_field (r, &r, &field))
       goto error;
     gst_structure_set_field (structure, &field);
-    while (*r && (g_ascii_isspace (*r) || (r[0] == '\\'
-                && g_ascii_isspace (r[1]))))
-      r++;
-  }
+
+  } while (TRUE);
 
   if (end)
     *end = (char *) string + (r - copy);
@@ -2045,23 +2240,23 @@ gst_structure_fixate_field_nearest_fraction (GstStructure * structure,
 
     gst_value_set_fraction (&target, target_numerator, target_denominator);
 
+    best = NULL;
+
     n = gst_value_list_get_size (value);
     for (i = 0; i < n; i++) {
       list_value = gst_value_list_get_value (value, i);
       if (G_VALUE_TYPE (list_value) == GST_TYPE_FRACTION) {
-        if (best == NULL) {
-          best = list_value;
-          gst_value_set_fraction (&best_diff, 0, 1);
-        } else {
-          if (gst_value_compare (list_value, &target) == GST_VALUE_LESS_THAN)
-            gst_value_fraction_subtract (&cur_diff, &target, list_value);
-          else
-            gst_value_fraction_subtract (&cur_diff, list_value, &target);
 
-          if (gst_value_compare (&cur_diff, &best_diff) == GST_VALUE_LESS_THAN) {
-            best = list_value;
-            g_value_copy (&cur_diff, &best_diff);
-          }
+        if (gst_value_compare (list_value, &target) == GST_VALUE_LESS_THAN)
+          gst_value_fraction_subtract (&cur_diff, &target, list_value);
+        else
+          gst_value_fraction_subtract (&cur_diff, list_value, &target);
+
+        if (!best
+            || gst_value_compare (&cur_diff,
+                &best_diff) == GST_VALUE_LESS_THAN) {
+          best = list_value;
+          g_value_copy (&cur_diff, &best_diff);
         }
       }
     }
@@ -2074,6 +2269,5 @@ gst_structure_fixate_field_nearest_fraction (GstStructure * structure,
     g_value_unset (&target);
     return res;
   }
-
   return FALSE;
 }
