@@ -43,6 +43,10 @@ GMutex *mutex;
 GCond *start_cond;              /* used to notify main thread of thread startups */
 GCond *sync_cond;               /* used to synchronize all threads and main thread */
 
+GList *buffers = NULL;
+GMutex *check_mutex = NULL;
+GCond *check_cond = NULL;
+
 gboolean _gst_check_debug = FALSE;
 gboolean _gst_check_raised_critical = FALSE;
 gboolean _gst_check_raised_warning = FALSE;
@@ -179,7 +183,7 @@ gst_check_setup_src_pad (GstElement * element,
       GST_ELEMENT_NAME (element));
   ASSERT_OBJECT_REFCOUNT (sinkpad, "sinkpad", 2);
   if (caps)
-    fail_unless (gst_pad_set_caps (srcpad, caps));
+    fail_unless (gst_pad_set_caps (srcpad, caps), "could not set caps on pad");
   fail_unless (gst_pad_link (srcpad, sinkpad) == GST_PAD_LINK_OK,
       "Could not link source and %s sink pads", GST_ELEMENT_NAME (element));
   gst_object_unref (sinkpad);   /* because we got it higher up */
@@ -231,7 +235,7 @@ gst_check_setup_sink_pad (GstElement * element, GstStaticPadTemplate * template,
   fail_if (srcpad == NULL, "Could not get source pad from %s",
       GST_ELEMENT_NAME (element));
   if (caps)
-    fail_unless (gst_pad_set_caps (sinkpad, caps));
+    fail_unless (gst_pad_set_caps (sinkpad, caps), "Could not set pad caps");
   gst_pad_set_chain_function (sinkpad, gst_check_chain_func);
 
   GST_DEBUG_OBJECT (element, "Linking element src pad and receiving sink pad");
@@ -329,3 +333,28 @@ gst_check_run_suite (Suite * suite, const gchar * name, const gchar * fname)
   srunner_free (sr);
   return nf;
 }
+
+gboolean
+_gst_check_run_test_func (const gchar * func_name)
+{
+  const gchar *gst_checks;
+  gboolean res = FALSE;
+  gchar **funcs, **f;
+
+  gst_checks = g_getenv ("GST_CHECKS");
+
+  /* no filter specified => run all checks */
+  if (gst_checks == NULL || *gst_checks == '\0')
+    return TRUE;
+
+  /* only run specified functions */
+  funcs = g_strsplit (gst_checks, ",", -1);
+  for (f = funcs; f != NULL && *f != NULL; ++f) {
+    if (strcmp (*f, func_name) == 0) {
+      res = TRUE;
+      break;
+    }
+  }
+  g_strfreev (funcs);
+  return res;
+}
