@@ -24,8 +24,7 @@
  * @short_description: read from arbitrary point in a file
  * @see_also: #GstFileSrc
  *
- * Read data from a file in the local file system. The implementation is using
- * mmap(2) to read chunks from the file in an efficient way.
+ * Read data from a file in the local file system.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -81,6 +80,9 @@ static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
  * GStreamer Default File Source
  * Theory of Operation
  *
+ * Update: see GstFileSrc:use-mmap property documentation below
+ *         for why use of mmap() is disabled by default.
+ *
  * This source uses mmap(2) to efficiently load data from a file.
  * To do this without seriously polluting the applications' memory
  * space, it must do so in smaller chunks, say 1-4MB at a time.
@@ -116,12 +118,6 @@ static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
 GST_DEBUG_CATEGORY_STATIC (gst_file_src_debug);
 #define GST_CAT_DEFAULT gst_file_src_debug
 
-static const GstElementDetails gst_file_src_details =
-GST_ELEMENT_DETAILS ("File Source",
-    "Source/File",
-    "Read from arbitrary point in a file",
-    "Erik Walthinsen <omega@cse.ogi.edu>");
-
 /* FileSrc signals and args */
 enum
 {
@@ -131,7 +127,9 @@ enum
 
 #define DEFAULT_BLOCKSIZE       4*1024
 #define DEFAULT_MMAPSIZE        4*1024*1024
-#define DEFAULT_TOUCH           FALSE
+#define DEFAULT_TOUCH           TRUE
+#define DEFAULT_USEMMAP         FALSE
+#define DEFAULT_SEQUENTIAL      FALSE
 
 enum
 {
@@ -139,7 +137,9 @@ enum
   ARG_LOCATION,
   ARG_FD,
   ARG_MMAPSIZE,
-  ARG_TOUCH
+  ARG_SEQUENTIAL,
+  ARG_TOUCH,
+  ARG_USEMMAP
 };
 
 static void gst_file_src_finalize (GObject * object);
@@ -182,10 +182,13 @@ gst_file_src_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
 
+  gst_element_class_set_details_simple (gstelement_class,
+      "File Source",
+      "Source/File",
+      "Read from arbitrary point in a file",
+      "Erik Walthinsen <omega@cse.ogi.edu>");
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&srctemplate));
-
-  gst_element_class_set_details (gstelement_class, &gst_file_src_details);
 }
 
 static void
@@ -214,8 +217,36 @@ gst_file_src_class_init (GstFileSrcClass * klass)
           "Size in bytes of mmap()d regions", 0, G_MAXULONG, DEFAULT_MMAPSIZE,
           G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_TOUCH,
-      g_param_spec_boolean ("touch", "Touch read data",
-          "Touch data to force disk read", DEFAULT_TOUCH, G_PARAM_READWRITE));
+      g_param_spec_boolean ("touch", "Touch mapped region read data",
+          "Touch mmapped data regions to force them to be read from disk",
+          DEFAULT_TOUCH, G_PARAM_READWRITE));
+  /**
+   * GstFileSrc:use-mmap
+   *
+   * Whether to use mmap(). Set to TRUE to force use of mmap() instead of
+   * read() for reading data.
+   *
+   * Use of mmap() is disabled by default since with mmap() there are a
+   * number of occasions where the process/application will be notified of
+   * read errors via a SIGBUS signal from the kernel, which will lead to
+   * the application being killed if not handled by the application. This
+   * is something that is difficult to work around for a library like
+   * GStreamer, hence use of mmap() is disabled by default. Said errors
+   * can occur for example when an external device (e.g. an external hard
+   * drive or a portable music player) are unplugged while in use, or when
+   * a CD/DVD medium cannot be be read because the medium is scratched or
+   * otherwise damaged.
+   *
+   **/
+  g_object_class_install_property (gobject_class, ARG_USEMMAP,
+      g_param_spec_boolean ("use-mmap", "Use mmap to read data",
+          "Whether to use mmap() instead of read()",
+          DEFAULT_USEMMAP, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, ARG_SEQUENTIAL,
+      g_param_spec_boolean ("sequential", "Optimise for sequential mmap access",
+          "Whether to use madvise to hint to the kernel that access to "
+          "mmap pages will be sequential",
+          DEFAULT_SEQUENTIAL, G_PARAM_READWRITE));
 
   gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_file_src_finalize);
 
@@ -226,7 +257,8 @@ gst_file_src_class_init (GstFileSrcClass * klass)
   gstbasesrc_class->create = GST_DEBUG_FUNCPTR (gst_file_src_create);
 
   if (sizeof (off_t) < 8) {
-    GST_LOG ("No large file support, sizeof (off_t) = %u!", sizeof (off_t));
+    GST_LOG ("No large file support, sizeof (off_t) = %" G_GSIZE_FORMAT "!",
+        sizeof (off_t));
   }
 }
 
@@ -245,6 +277,8 @@ gst_file_src_init (GstFileSrc * src, GstFileSrcClass * g_class)
 
   src->mapbuf = NULL;
   src->mapsize = DEFAULT_MMAPSIZE;      /* default is 4MB */
+  src->use_mmap = DEFAULT_USEMMAP;
+  src->sequential = DEFAULT_SEQUENTIAL;
 
   src->is_regular = FALSE;
 }
@@ -265,16 +299,14 @@ gst_file_src_finalize (GObject * object)
 static gboolean
 gst_file_src_set_location (GstFileSrc * src, const gchar * location)
 {
-  /* the element must be stopped in order to do this */
-  GST_STATE_LOCK (src);
-  {
-    GstState state;
+  GstState state;
 
-    state = GST_STATE (src);
-    if (state != GST_STATE_READY && state != GST_STATE_NULL)
-      goto wrong_state;
-  }
-  GST_STATE_UNLOCK (src);
+  /* the element must be stopped in order to do this */
+  GST_OBJECT_LOCK (src);
+  state = GST_STATE (src);
+  if (state != GST_STATE_READY && state != GST_STATE_NULL)
+    goto wrong_state;
+  GST_OBJECT_UNLOCK (src);
 
   g_free (src->filename);
   g_free (src->uri);
@@ -296,7 +328,7 @@ gst_file_src_set_location (GstFileSrc * src, const gchar * location)
 wrong_state:
   {
     GST_DEBUG_OBJECT (src, "setting location in wrong state");
-    GST_STATE_UNLOCK (src);
+    GST_OBJECT_UNLOCK (src);
     return FALSE;
   }
 }
@@ -318,7 +350,6 @@ gst_file_src_set_property (GObject * object, guint prop_id,
     case ARG_MMAPSIZE:
       if ((src->mapsize % src->pagesize) == 0) {
         src->mapsize = g_value_get_ulong (value);
-        g_object_notify (G_OBJECT (src), "mmapsize");
       } else {
         GST_INFO_OBJECT (src,
             "invalid mapsize, must be a multiple of pagesize, which is %d",
@@ -327,7 +358,12 @@ gst_file_src_set_property (GObject * object, guint prop_id,
       break;
     case ARG_TOUCH:
       src->touch = g_value_get_boolean (value);
-      g_object_notify (G_OBJECT (src), "touch");
+      break;
+    case ARG_SEQUENTIAL:
+      src->sequential = g_value_get_boolean (value);
+      break;
+    case ARG_USEMMAP:
+      src->use_mmap = g_value_get_boolean (value);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -358,6 +394,12 @@ gst_file_src_get_property (GObject * object, guint prop_id, GValue * value,
     case ARG_TOUCH:
       g_value_set_boolean (value, src->touch);
       break;
+    case ARG_SEQUENTIAL:
+      g_value_set_boolean (value, src->sequential);
+      break;
+    case ARG_USEMMAP:
+      g_value_set_boolean (value, src->use_mmap);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -466,13 +508,13 @@ gst_mmap_buffer_finalize (GstMmapBuffer * mmap_buffer)
 #ifdef MADV_DONTNEED
   /* madvise to tell the kernel what to do with it */
   if (madvise (data, size, MADV_DONTNEED) < 0) {
-    GST_WARNING_OBJECT (src, "warning: madvise failed: %s", strerror (errno));
+    GST_WARNING_OBJECT (src, "warning: madvise failed: %s", g_strerror (errno));
   }
 #endif
 
   /* now unmap the memory */
   if (munmap (data, size) < 0) {
-    GST_WARNING_OBJECT (src, "warning: munmap failed: %s", strerror (errno));
+    GST_WARNING_OBJECT (src, "warning: munmap failed: %s", g_strerror (errno));
   }
 
   /* cast to unsigned long, since there's no gportable way to print
@@ -508,9 +550,12 @@ gst_file_src_map_region (GstFileSrc * src, off_t offset, size_t size,
   GST_MMAP_BUFFER (buf)->filesrc = src;
 
 #ifdef MADV_SEQUENTIAL
-  /* madvise to tell the kernel what to do with it */
-  if (madvise (mmapregion, size, MADV_SEQUENTIAL) < 0) {
-    GST_WARNING_OBJECT (src, "warning: madvise failed: %s", strerror (errno));
+  if (src->sequential) {
+    /* madvise to tell the kernel what to do with it */
+    if (madvise (mmapregion, size, MADV_SEQUENTIAL) < 0) {
+      GST_WARNING_OBJECT (src, "warning: madvise failed: %s",
+          g_strerror (errno));
+    }
   }
 #endif
 
@@ -528,7 +573,7 @@ mmap_failed:
     if (!testonly) {
       GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
           ("mmap (0x%08lx, %d, 0x%llx) failed: %s",
-              (gulong) size, src->fd, offset, strerror (errno)));
+              (gulong) size, src->fd, offset, g_strerror (errno)));
     }
     return NULL;
   }
@@ -542,8 +587,8 @@ gst_file_src_map_small_region (GstFileSrc * src, off_t offset, size_t size)
   guint pagesize;
 
   GST_LOG_OBJECT (src,
-      "attempting to map a small buffer at %llu+%d",
-      (unsigned long long) offset, (gint) size);
+      "attempting to map a small buffer at %" G_GUINT64_FORMAT "+%d",
+      (guint64) offset, (gint) size);
 
   pagesize = src->pagesize;
 
@@ -559,8 +604,8 @@ gst_file_src_map_small_region (GstFileSrc * src, off_t offset, size_t size)
     mapsize = ((size + mod + pagesize - 1) / pagesize) * pagesize;
 
     GST_LOG_OBJECT (src,
-        "not on page boundaries, resizing to map to %llu+%d",
-        (unsigned long long) mapbase, (gint) mapsize);
+        "not on page boundaries, resizing to map to %" G_GUINT64_FORMAT "+%d",
+        (guint64) mapbase, (gint) mapsize);
 
     map = gst_file_src_map_region (src, mapbase, mapsize, FALSE);
     if (map == NULL)
@@ -603,18 +648,17 @@ gst_file_src_create_mmap (GstFileSrc * src, guint64 offset, guint length,
     /* if the end is before the mapend, the buffer is in current mmap region... */
     /* ('cause by definition if readend is in the buffer, so's readstart) */
     if (readend <= mapend) {
-      GST_LOG_OBJECT (src,
-          "read buf %llu+%d lives in current mapbuf %lld+%d, creating subbuffer of mapbuf",
-          offset, (int) readsize, mapstart, mapsize);
+      GST_LOG_OBJECT (src, "read buf %" G_GUINT64_FORMAT "+%u lives in "
+          "current mapbuf %u+%u, creating subbuffer of mapbuf",
+          offset, (guint) readsize, (guint) mapstart, (guint) mapsize);
       buf = gst_buffer_create_sub (src->mapbuf, offset - mapstart, readsize);
       GST_BUFFER_OFFSET (buf) = offset;
 
       /* if the start actually is within the current mmap region, map an overlap buffer */
     } else if (offset < mapend) {
-      GST_LOG_OBJECT (src,
-          "read buf %llu+%d starts in mapbuf %d+%d but ends outside, creating new mmap",
-          (unsigned long long) offset, (gint) readsize, (gint) mapstart,
-          (gint) mapsize);
+      GST_LOG_OBJECT (src, "read buf %" G_GUINT64_FORMAT "+%u starts in "
+          "mapbuf %u+%u but ends outside, creating new mmap",
+          offset, (guint) readsize, (guint) mapstart, (guint) mapsize);
       buf = gst_file_src_map_small_region (src, offset, readsize);
       if (buf == NULL)
         goto could_not_mmap;
@@ -628,10 +672,9 @@ gst_file_src_create_mmap (GstFileSrc * src, guint64 offset, guint length,
     /* either the read buffer overlaps the start of the mmap region */
     /* or the read buffer fully contains the current mmap region    */
     /* either way, it's really not relevant, we just create a new region anyway */
-    GST_LOG_OBJECT (src,
-        "read buf %llu+%d starts before mapbuf %d+%d, but overlaps it",
-        (unsigned long long) offset, (gint) readsize, (gint) mapstart,
-        (gint) mapsize);
+    GST_LOG_OBJECT (src, "read buf %" G_GUINT64_FORMAT "+%d starts before "
+        "mapbuf %d+%d, but overlaps it", (guint64) offset, (gint) readsize,
+        (gint) mapstart, (gint) mapsize);
     buf = gst_file_src_map_small_region (src, offset, readsize);
     if (buf == NULL)
       goto could_not_mmap;
@@ -658,9 +701,9 @@ gst_file_src_create_mmap (GstFileSrc * src, guint64 offset, guint length,
 
       off_t nextmap = offset - (offset % src->mapsize);
 
-      GST_LOG_OBJECT (src,
-          "read buf %llu+%d in new mapbuf at %llu+%d, mapping and subbuffering",
-          offset, readsize, nextmap, src->mapsize);
+      GST_LOG_OBJECT (src, "read buf %" G_GUINT64_FORMAT "+%d in new mapbuf "
+          "at %" G_GUINT64_FORMAT "+%d, mapping and subbuffering",
+          offset, (gint) readsize, (guint64) nextmap, (gint) src->mapsize);
       /* first, we're done with the old mapbuf */
       gst_buffer_unref (src->mapbuf);
       mapsize = src->mapsize;
@@ -668,7 +711,7 @@ gst_file_src_create_mmap (GstFileSrc * src, guint64 offset, guint length,
       /* double the mapsize as long as the readsize is smaller */
       while (readsize + offset > nextmap + mapsize) {
         GST_LOG_OBJECT (src, "readsize smaller then mapsize %08x %d",
-            readsize, (int) mapsize);
+            (guint) readsize, (gint) mapsize);
         mapsize <<= 1;
       }
       /* create a new one */
@@ -723,11 +766,11 @@ gst_file_src_create_read (GstFileSrc * src, guint64 offset, guint length,
   int ret;
   GstBuffer *buf;
 
-  if (src->read_position != offset) {
+  if (G_UNLIKELY (src->read_position != offset)) {
     off_t res;
 
     res = lseek (src->fd, offset, SEEK_SET);
-    if (res < 0 || res != offset)
+    if (G_UNLIKELY (res < 0 || res != offset))
       goto seek_failed;
 
     src->read_position = offset;
@@ -737,19 +780,17 @@ gst_file_src_create_read (GstFileSrc * src, guint64 offset, guint length,
 
   GST_LOG_OBJECT (src, "Reading %d bytes", length);
   ret = read (src->fd, GST_BUFFER_DATA (buf), length);
-  if (ret < 0)
+  if (G_UNLIKELY (ret < 0))
     goto could_not_read;
 
-  /* regular files should have given us what we expected */
-  if ((guint) ret < length && src->is_regular)
+  /* seekable regular files should have given us what we expected */
+  if (G_UNLIKELY ((guint) ret < length && src->seekable))
     goto unexpected_eos;
 
-  /* other files should eos if they read 0 */
-  if (ret == 0) {
-    GST_DEBUG ("non-regular file hits EOS");
-    gst_buffer_unref (buf);
-    return GST_FLOW_UNEXPECTED;
-  }
+  /* other files should eos if they read 0 and more was requested */
+  if (G_UNLIKELY (ret == 0 && length > 0))
+    goto eos;
+
   length = ret;
 
   GST_BUFFER_SIZE (buf) = length;
@@ -781,6 +822,12 @@ unexpected_eos:
     gst_buffer_unref (buf);
     return GST_FLOW_ERROR;
   }
+eos:
+  {
+    GST_DEBUG ("non-regular file hits EOS");
+    gst_buffer_unref (buf);
+    return GST_FLOW_UNEXPECTED;
+  }
 }
 
 static GstFlowReturn
@@ -876,27 +923,31 @@ gst_file_src_start (GstBaseSrc * basesrc)
     src->is_regular = TRUE;
 
 #ifdef HAVE_MMAP
-  /* FIXME: maybe we should only try to mmap if it's a regular file */
-  /* allocate the first mmap'd region if it's a regular file ? */
-  src->mapbuf = gst_file_src_map_region (src, 0, src->mapsize, TRUE);
-  if (src->mapbuf != NULL) {
-    GST_DEBUG_OBJECT (src, "using mmap for file");
-    src->using_mmap = TRUE;
-    src->seekable = TRUE;
-  } else
+  if (src->use_mmap) {
+    /* FIXME: maybe we should only try to mmap if it's a regular file */
+    /* allocate the first mmap'd region if it's a regular file ? */
+    src->mapbuf = gst_file_src_map_region (src, 0, src->mapsize, TRUE);
+    if (src->mapbuf != NULL) {
+      GST_DEBUG_OBJECT (src, "using mmap for file");
+      src->using_mmap = TRUE;
+      src->seekable = TRUE;
+    }
+  }
+  if (src->mapbuf == NULL)
 #endif
   {
     /* If not in mmap mode, we need to check if the underlying file is
      * seekable. */
-    off_t res = lseek (src->fd, 0, SEEK_CUR);
+    off_t res = lseek (src->fd, 0, SEEK_END);
 
     if (res < 0) {
       GST_LOG_OBJECT (src, "disabling seeking, not in mmap mode and lseek "
-          "failed: %s", strerror (errno));
+          "failed: %s", g_strerror (errno));
       src->seekable = FALSE;
     } else {
       src->seekable = TRUE;
     }
+    lseek (src->fd, 0, SEEK_SET);
   }
 
   /* We can only really do seeking on regular files - for other file types, we
@@ -921,8 +972,8 @@ open_failed:
         break;
       default:
         GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
-            (_("Could not open file \"%s\" for reading: %s."), src->filename,
-                strerror (errno)), GST_ERROR_SYSTEM);
+            (_("Could not open file \"%s\" for reading."), src->filename),
+            GST_ERROR_SYSTEM);
         break;
     }
     return FALSE;
@@ -930,7 +981,7 @@ open_failed:
 no_stat:
   {
     GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
-        (_("could not get info on \"%s\"."), src->filename), (NULL));
+        (_("Could not get info on \"%s\"."), src->filename), (NULL));
     close (src->fd);
     return FALSE;
   }
@@ -973,7 +1024,7 @@ gst_file_src_stop (GstBaseSrc * basesrc)
 
 /*** GSTURIHANDLER INTERFACE *************************************************/
 
-static guint
+static GstURIType
 gst_file_src_uri_get_type (void)
 {
   return GST_URI_SRC;
@@ -1006,7 +1057,35 @@ gst_file_src_uri_set_uri (GstURIHandler * handler, const gchar * uri)
     return FALSE;
   }
   g_free (protocol);
-  location = gst_uri_get_location (uri);
+
+  /* allow file://localhost/foo/bar by stripping localhost but fail
+   * for every other hostname */
+  if (g_str_has_prefix (uri, "file://localhost/")) {
+    char *tmp;
+
+    /* 16 == strlen ("file://localhost") */
+    tmp = g_strconcat ("file://", uri + 16, NULL);
+    /* we use gst_uri_get_location() although we already have the
+     * "location" with uri + 16 because it provides unescaping */
+    location = gst_uri_get_location (tmp);
+    g_free (tmp);
+  } else if (strcmp (uri, "file://") == 0) {
+    /* Special case for "file://" as this is used by some applications
+     *  to test with gst_element_make_from_uri if there's an element
+     *  that supports the URI protocol. */
+    gst_file_src_set_location (src, NULL);
+    return TRUE;
+  } else {
+    location = gst_uri_get_location (uri);
+  }
+
+  if (!location)
+    return FALSE;
+  if (!g_path_is_absolute (location)) {
+    g_free (location);
+    return FALSE;
+  }
+
   ret = gst_file_src_set_location (src, location);
   g_free (location);
 
