@@ -63,6 +63,12 @@ struct _GstQueueSize {
     guint64 time;
 };
 
+#define GST_QUEUE_CLEAR_LEVEL(l) G_STMT_START {         \
+  l.buffers = 0;                                        \
+  l.bytes = 0;                                          \
+  l.time = 0;                                           \
+} G_STMT_END
+
 /**
  * GstQueue:
  *
@@ -75,8 +81,14 @@ struct _GstQueue {
   GstPad *sinkpad;
   GstPad *srcpad;
 
+  /* segments to keep track of timestamps */
+  GstSegment sink_segment;
+  GstSegment src_segment;
+
   /* flowreturn when srcpad is paused */
   GstFlowReturn srcresult;
+  gboolean      unexpected;
+  gboolean      eos;
 
   /* the queue of data we're keeping our grubby hands on */
   GQueue *queue;
@@ -93,8 +105,6 @@ struct _GstQueue {
   GMutex *qlock;	/* lock for queue (vs object lock) */
   GCond *item_add;	/* signals buffers now available for reading */
   GCond *item_del;	/* signals space now available for writing */
-
-  gpointer _gst_reserved[GST_PADDING];
 };
 
 struct _GstQueueClass {
@@ -106,7 +116,7 @@ struct _GstQueueClass {
   void (*running)	(GstQueue *queue);
   void (*overrun)	(GstQueue *queue);
 
-  gpointer _gst_reserved[GST_PADDING];
+  void (*pushing)	(GstQueue *queue);
 };
 
 GType gst_queue_get_type (void);
