@@ -1,5 +1,6 @@
 /* GStreamer
  * Copyright (C) <2005> Thomas Vander Stichele <thomas at apestaart dot org>
+ * Copyright (C) <2006> Tim-Philipp Müller <tim centricular net>
  *
  * gstutils.c: Unit test for functions in gstutils
  *
@@ -282,9 +283,9 @@ GST_END_TEST;
 
 GST_START_TEST (test_guint64_to_gdouble)
 {
-  guint64 from[] = { 0, 1, 100, 10000, G_GINT64_CONSTANT (1) << 63,
-    (G_GINT64_CONSTANT (1) << 63) + 1,
-    (G_GINT64_CONSTANT (1) << 63) + (G_GINT64_CONSTANT (1) << 62)
+  guint64 from[] = { 0, 1, 100, 10000, (guint64) (1) << 63,
+    ((guint64) (1) << 63) + 1,
+    ((guint64) (1) << 63) + (G_GINT64_CONSTANT (1) << 62)
   };
   gdouble to[] = { 0., 1., 100., 10000., 9223372036854775808.,
     9223372036854775809., 13835058055282163712.
@@ -311,9 +312,9 @@ GST_START_TEST (test_gdouble_to_guint64)
   gdouble from[] = { 0., 1., 100., 10000., 9223372036854775808.,
     9223372036854775809., 13835058055282163712.
   };
-  guint64 to[] = { 0, 1, 100, 10000, G_GINT64_CONSTANT (1) << 63,
-    (G_GINT64_CONSTANT (1) << 63) + 1,
-    (G_GINT64_CONSTANT (1) << 63) + (G_GINT64_CONSTANT (1) << 62)
+  guint64 to[] = { 0, 1, 100, 10000, (guint64) (1) << 63,
+    ((guint64) (1) << 63) + 1,
+    ((guint64) (1) << 63) + (G_GINT64_CONSTANT (1) << 62)
   };
   guint64 tolerance[] = { 0, 0, 0, 0, 0, 1, 1 };
   gint i;
@@ -332,6 +333,7 @@ GST_START_TEST (test_gdouble_to_guint64)
 
 GST_END_TEST;
 
+#ifndef GST_DISABLE_PARSE
 GST_START_TEST (test_parse_bin_from_description)
 {
   struct
@@ -405,9 +407,187 @@ GST_START_TEST (test_parse_bin_from_description)
           bin_tests[i].pad_names, s->str, bin_tests[i].bin_desc);
     }
     g_string_free (s, TRUE);
+
+    gst_object_unref (bin);
   }
 }
 
+GST_END_TEST;
+#endif
+
+GST_START_TEST (test_element_found_tags)
+{
+  GstElement *pipeline, *fakesrc, *fakesink;
+  GstTagList *list;
+  GstBus *bus;
+  GstMessage *message;
+
+  pipeline = gst_element_factory_make ("pipeline", NULL);
+  fakesrc = gst_element_factory_make ("fakesrc", NULL);
+  fakesink = gst_element_factory_make ("fakesink", NULL);
+  list = gst_tag_list_new ();
+
+  g_object_set (fakesrc, "num-buffers", (int) 10, NULL);
+
+  gst_bin_add_many (GST_BIN (pipeline), fakesrc, fakesink, NULL);
+  gst_element_link (fakesrc, fakesink);
+
+  gst_element_set_state (pipeline, GST_STATE_PLAYING);
+
+  gst_element_found_tags (GST_ELEMENT (fakesrc), list);
+
+  bus = gst_element_get_bus (pipeline);
+  message = gst_bus_poll (bus, GST_MESSAGE_EOS, -1);
+  gst_message_unref (message);
+  gst_object_unref (bus);
+
+  /* FIXME: maybe also check if the fakesink receives the message */
+
+  gst_element_set_state (pipeline, GST_STATE_NULL);
+  gst_object_unref (pipeline);
+}
+
+GST_END_TEST;
+
+GST_START_TEST (test_element_unlink)
+{
+  GstElement *src, *sink;
+
+  src = gst_element_factory_make ("fakesrc", NULL);
+  sink = gst_element_factory_make ("fakesink", NULL);
+  fail_unless (gst_element_link (src, sink) != FALSE);
+  gst_element_unlink (src, sink);
+  gst_object_unref (src);
+  gst_object_unref (sink);
+}
+
+GST_END_TEST;
+
+GST_START_TEST (test_set_value_from_string)
+{
+  GValue val = { 0, };
+
+  /* g_return_if_fail */
+  ASSERT_CRITICAL (gst_util_set_value_from_string (NULL, "xyz"));
+
+  g_value_init (&val, G_TYPE_STRING);
+  ASSERT_CRITICAL (gst_util_set_value_from_string (&val, NULL));
+  g_value_unset (&val);
+
+  /* string => string */
+  g_value_init (&val, G_TYPE_STRING);
+  gst_util_set_value_from_string (&val, "Y00");
+  fail_unless (g_value_get_string (&val) != NULL);
+  fail_unless_equals_string (g_value_get_string (&val), "Y00");
+  g_value_unset (&val);
+
+  /* string => int */
+  g_value_init (&val, G_TYPE_INT);
+  gst_util_set_value_from_string (&val, "987654321");
+  fail_unless (g_value_get_int (&val) == 987654321);
+  g_value_unset (&val);
+
+  g_value_init (&val, G_TYPE_INT);
+  ASSERT_CRITICAL (gst_util_set_value_from_string (&val, "xyz"));
+  g_value_unset (&val);
+
+  /* string => uint */
+  g_value_init (&val, G_TYPE_UINT);
+  gst_util_set_value_from_string (&val, "987654321");
+  fail_unless (g_value_get_uint (&val) == 987654321);
+  g_value_unset (&val);
+
+  /* CHECKME: is this really desired behaviour? (tpm) */
+  g_value_init (&val, G_TYPE_UINT);
+  gst_util_set_value_from_string (&val, "-999");
+  fail_unless (g_value_get_uint (&val) == ((guint) 0 - (guint) 999));
+  g_value_unset (&val);
+
+  g_value_init (&val, G_TYPE_UINT);
+  ASSERT_CRITICAL (gst_util_set_value_from_string (&val, "xyz"));
+  g_value_unset (&val);
+
+  /* string => long */
+  g_value_init (&val, G_TYPE_LONG);
+  gst_util_set_value_from_string (&val, "987654321");
+  fail_unless (g_value_get_long (&val) == 987654321);
+  g_value_unset (&val);
+
+  g_value_init (&val, G_TYPE_LONG);
+  ASSERT_CRITICAL (gst_util_set_value_from_string (&val, "xyz"));
+  g_value_unset (&val);
+
+  /* string => ulong */
+  g_value_init (&val, G_TYPE_ULONG);
+  gst_util_set_value_from_string (&val, "987654321");
+  fail_unless (g_value_get_ulong (&val) == 987654321);
+  g_value_unset (&val);
+
+  /* CHECKME: is this really desired behaviour? (tpm) */
+  g_value_init (&val, G_TYPE_ULONG);
+  gst_util_set_value_from_string (&val, "-999");
+  fail_unless (g_value_get_ulong (&val) == ((gulong) 0 - (gulong) 999));
+  g_value_unset (&val);
+
+  g_value_init (&val, G_TYPE_ULONG);
+  ASSERT_CRITICAL (gst_util_set_value_from_string (&val, "xyz"));
+  g_value_unset (&val);
+
+  /* string => boolean */
+  g_value_init (&val, G_TYPE_BOOLEAN);
+  gst_util_set_value_from_string (&val, "true");
+  fail_unless_equals_int (g_value_get_boolean (&val), TRUE);
+  g_value_unset (&val);
+
+  g_value_init (&val, G_TYPE_BOOLEAN);
+  gst_util_set_value_from_string (&val, "TRUE");
+  fail_unless_equals_int (g_value_get_boolean (&val), TRUE);
+  g_value_unset (&val);
+
+  g_value_init (&val, G_TYPE_BOOLEAN);
+  gst_util_set_value_from_string (&val, "false");
+  fail_unless_equals_int (g_value_get_boolean (&val), FALSE);
+  g_value_unset (&val);
+
+  g_value_init (&val, G_TYPE_BOOLEAN);
+  gst_util_set_value_from_string (&val, "FALSE");
+  fail_unless_equals_int (g_value_get_boolean (&val), FALSE);
+  g_value_unset (&val);
+
+  g_value_init (&val, G_TYPE_BOOLEAN);
+  gst_util_set_value_from_string (&val, "bleh");
+  fail_unless_equals_int (g_value_get_boolean (&val), FALSE);
+  g_value_unset (&val);
+
+#if 0
+  /* string => float (yay, localisation issues involved) */
+  g_value_init (&val, G_TYPE_FLOAT);
+  gst_util_set_value_from_string (&val, "987.654");
+  fail_unless (g_value_get_float (&val) >= 987.653 &&
+      g_value_get_float (&val) <= 987.655);
+  g_value_unset (&val);
+
+  g_value_init (&val, G_TYPE_FLOAT);
+  gst_util_set_value_from_string (&val, "987,654");
+  fail_unless (g_value_get_float (&val) >= 987.653 &&
+      g_value_get_float (&val) <= 987.655);
+  g_value_unset (&val);
+
+  /* string => double (yay, localisation issues involved) */
+  g_value_init (&val, G_TYPE_DOUBLE);
+  gst_util_set_value_from_string (&val, "987.654");
+  fail_unless (g_value_get_double (&val) >= 987.653 &&
+      g_value_get_double (&val) <= 987.655);
+  g_value_unset (&val);
+
+  g_value_init (&val, G_TYPE_DOUBLE);
+  gst_util_set_value_from_string (&val, "987,654");
+  fail_unless (g_value_get_double (&val) >= 987.653 &&
+      g_value_get_double (&val) <= 987.655);
+  g_value_unset (&val);
+#endif
+}
+
 GST_END_TEST;
 
 Suite *
@@ -424,23 +604,13 @@ gst_utils_suite (void)
   tcase_add_test (tc_chain, test_math_scale_random);
   tcase_add_test (tc_chain, test_guint64_to_gdouble);
   tcase_add_test (tc_chain, test_gdouble_to_guint64);
+#ifndef GST_DISABLE_PARSE
   tcase_add_test (tc_chain, test_parse_bin_from_description);
+#endif
+  tcase_add_test (tc_chain, test_element_found_tags);
+  tcase_add_test (tc_chain, test_element_unlink);
+  tcase_add_test (tc_chain, test_set_value_from_string);
   return s;
 }
 
-int
-main (int argc, char **argv)
-{
-  int nf;
-
-  Suite *s = gst_utils_suite ();
-  SRunner *sr = srunner_create (s);
-
-  gst_check_init (&argc, &argv);
-
-  srunner_run_all (sr, CK_NORMAL);
-  nf = srunner_ntests_failed (sr);
-  srunner_free (sr);
-
-  return nf;
-}
+GST_CHECK_MAIN (gst_utils);
