@@ -60,6 +60,7 @@ static GstElement *pipeline;
 static gboolean caught_error = FALSE;
 static gboolean tags = FALSE;
 static gboolean messages = FALSE;
+static gboolean is_live = FALSE;
 
 
 #ifndef GST_DISABLE_LOADSAVE
@@ -449,6 +450,10 @@ event_loop (GstElement * pipeline, gboolean blocking, GstState target_state)
         gchar *debug;
         gchar *name = gst_object_get_path_string (GST_MESSAGE_SRC (message));
 
+        /* dump graph on warning */
+        GST_DEBUG_BIN_TO_DOT_FILE_WITH_TS (GST_BIN (pipeline),
+            GST_DEBUG_GRAPH_SHOW_ALL, "gst-launch.warning");
+
         gst_message_parse_warning (message, &gerror, &debug);
         g_print (_("WARNING: from element %s: %s\n"), name, gerror->message);
         if (debug) {
@@ -463,6 +468,10 @@ event_loop (GstElement * pipeline, gboolean blocking, GstState target_state)
         GError *gerror;
         gchar *debug;
 
+        /* dump graph on error */
+        GST_DEBUG_BIN_TO_DOT_FILE_WITH_TS (GST_BIN (pipeline),
+            GST_DEBUG_GRAPH_SHOW_ALL, "gst-launch.error");
+
         gst_message_parse_error (message, &gerror, &debug);
         gst_object_default_error (GST_MESSAGE_SRC (message), gerror, debug);
         g_error_free (gerror);
@@ -476,10 +485,24 @@ event_loop (GstElement * pipeline, gboolean blocking, GstState target_state)
 
         gst_message_parse_state_changed (message, &old, &new, &pending);
 
+        /* debug each state change
+           GST_DEBUG_BIN_TO_DOT_FILE_WITH_TS (GST_BIN (pipeline), GST_DEBUG_GRAPH_SHOW_ALL, "gst-launch");
+         */
+
         /* we only care about pipeline state change messages */
         if (GST_MESSAGE_SRC (message) != GST_OBJECT_CAST (pipeline))
           break;
 
+        /* debug only overall state changes
+           {
+           gchar *dump_name;
+
+           dump_name = g_strdup_printf ("gst-launch.%s",gst_element_state_get_name (new);
+           GST_DEBUG_BIN_TO_DOT_FILE_WITH_TS (GST_BIN (pipeline), GST_DEBUG_GRAPH_SHOW_ALL, dump_name);
+           g_free (dump_name);
+           }
+         */
+
         /* ignore when we are buffering since then we mess with the states
          * ourselves. */
         if (buffering) {
@@ -499,7 +522,11 @@ event_loop (GstElement * pipeline, gboolean blocking, GstState target_state)
         gint percent;
 
         gst_message_parse_buffering (message, &percent);
-        fprintf (stderr, "buffering... %d  \r", percent);
+        fprintf (stderr, _("buffering... %d  \r"), percent);
+
+        /* no state management needed for live pipelines */
+        if (is_live)
+          break;
 
         if (percent == 100) {
           /* a 100% message means buffering is done */
@@ -530,8 +557,7 @@ event_loop (GstElement * pipeline, gboolean blocking, GstState target_state)
         if (gst_structure_has_name (s, "GstLaunchInterrupt")) {
           /* this application message is posted when we caught an interrupt and
            * we need to stop the pipeline. */
-          fprintf (stderr, _("Interrupt: Setting pipeline to PAUSED ...\n"));
-          gst_element_set_state (pipeline, GST_STATE_PAUSED);
+          fprintf (stderr, _("Interrupt: Stopping pipeline ...\n"));
           /* return TRUE when we caught an interrupt */
           res = TRUE;
           goto exit;
@@ -558,8 +584,6 @@ exit:
 int
 main (int argc, char *argv[])
 {
-  gint i, j;
-
   /* options */
   gboolean verbose = FALSE;
   gboolean no_fault = FALSE;
@@ -619,25 +643,6 @@ main (int argc, char *argv[])
 
   gst_tools_print_version ("gst-launch");
 
-  /* FIXpopt: strip short args, too. We do it ourselves for now */
-  j = 1;
-  for (i = 1; i < argc; i++) {
-    if (*(argv[i]) == '-') {
-      if (strlen (argv[i]) == 2) {
-        gchar *c = argv[i];
-
-        c++;
-        if (*c == 'X' || *c == 'o') {
-          i++;
-        }
-      }
-    } else {
-      argv[j] = argv[i];
-      j++;
-    }
-  }
-  argc = j;
-
 #ifndef DISABLE_FAULT_HANDLER
   if (!no_fault)
     fault_setup ();
@@ -721,6 +726,7 @@ main (int argc, char *argv[])
         goto end;
       case GST_STATE_CHANGE_NO_PREROLL:
         fprintf (stderr, _("Pipeline is live and does not need PREROLL ...\n"));
+        is_live = TRUE;
         break;
       case GST_STATE_CHANGE_ASYNC:
         fprintf (stderr, _("Pipeline is PREROLLING ...\n"));
@@ -741,22 +747,37 @@ main (int argc, char *argv[])
     if (caught_error) {
       fprintf (stderr, _("ERROR: pipeline doesn't want to preroll.\n"));
     } else {
-      GTimeVal tfthen, tfnow;
+      GstClockTime tfthen, tfnow;
       GstClockTimeDiff diff;
 
       fprintf (stderr, _("Setting pipeline to PLAYING ...\n"));
       if (gst_element_set_state (pipeline,
               GST_STATE_PLAYING) == GST_STATE_CHANGE_FAILURE) {
+        GstMessage *err_msg;
+        GstBus *bus;
+
         fprintf (stderr, _("ERROR: pipeline doesn't want to play.\n"));
+        bus = gst_element_get_bus (pipeline);
+        if ((err_msg = gst_bus_poll (bus, GST_MESSAGE_ERROR, 0))) {
+          GError *gerror;
+          gchar *debug;
+
+          gst_message_parse_error (err_msg, &gerror, &debug);
+          gst_object_default_error (GST_MESSAGE_SRC (err_msg), gerror, debug);
+          gst_message_unref (err_msg);
+          g_error_free (gerror);
+          g_free (debug);
+        }
+        gst_object_unref (bus);
         res = -1;
         goto end;
       }
 
-      g_get_current_time (&tfthen);
+      tfthen = gst_util_get_timestamp ();
       caught_error = event_loop (pipeline, TRUE, GST_STATE_PLAYING);
-      g_get_current_time (&tfnow);
+      tfnow = gst_util_get_timestamp ();
 
-      diff = GST_TIMEVAL_TO_TIME (tfnow) - GST_TIMEVAL_TO_TIME (tfthen);
+      diff = GST_CLOCK_DIFF (tfthen, tfnow);
 
       g_print (_("Execution ended after %" G_GUINT64_FORMAT " ns.\n"), diff);
     }
@@ -766,7 +787,8 @@ main (int argc, char *argv[])
 
     fprintf (stderr, _("Setting pipeline to PAUSED ...\n"));
     gst_element_set_state (pipeline, GST_STATE_PAUSED);
-    gst_element_get_state (pipeline, &state, &pending, GST_CLOCK_TIME_NONE);
+    if (!caught_error)
+      gst_element_get_state (pipeline, &state, &pending, GST_CLOCK_TIME_NONE);
     fprintf (stderr, _("Setting pipeline to READY ...\n"));
     gst_element_set_state (pipeline, GST_STATE_READY);
     gst_element_get_state (pipeline, &state, &pending, GST_CLOCK_TIME_NONE);
